@@ -778,6 +778,17 @@ ocsd_datapath_resp_t TrcPktDecodeEtmV4I::outputEvent(TrcStackElemParam *pParamEl
     return outputTraceElementIdx(pParamElem->getRootIndex(),m_output_elem);
 }
 
+ocsd_datapath_resp_t TrcPktDecodeEtmV4I::outputTraceRange(const bool executed, ocsd_trc_index_t index)
+{
+    m_output_elem.setType(OCSD_GEN_TRC_ELEM_INSTR_RANGE);
+    m_output_elem.setLastInstrInfo(executed, m_instr_info.type, m_instr_info.sub_type, m_instr_info.instr_size);
+    m_output_elem.setISA(m_instr_info.isa);
+    m_output_elem.setLastInstrCond(m_instr_info.is_conditional);
+    if (executed)
+        m_instr_info.isa = m_instr_info.next_isa;
+    return outputTraceElementIdx(index, m_output_elem);
+}
+
 ocsd_datapath_resp_t TrcPktDecodeEtmV4I::processAtom(const ocsd_atm_val atom, bool &bCont)
 {
     ocsd_datapath_resp_t resp = OCSD_RESP_CONT;
@@ -834,10 +845,7 @@ ocsd_datapath_resp_t TrcPktDecodeEtmV4I::processAtom(const ocsd_atm_val atom, bo
             }
             break;
         }
-        m_output_elem.setType(OCSD_GEN_TRC_ELEM_INSTR_RANGE);
-        m_output_elem.setLastInstrInfo((atom == ATOM_E),m_instr_info.type, m_instr_info.sub_type, m_instr_info.instr_size);
-        m_output_elem.setISA(m_instr_info.isa);
-        resp = outputTraceElementIdx(pElem->getRootIndex(),m_output_elem);
+        resp = outputTraceRange((atom == ATOM_E), pElem->getRootIndex());
 
     }
     else
@@ -848,10 +856,7 @@ ocsd_datapath_resp_t TrcPktDecodeEtmV4I::processAtom(const ocsd_atm_val atom, bo
         if(m_output_elem.st_addr != m_output_elem.en_addr)
         {
             // some trace before we were out of memory access range
-            m_output_elem.setType(OCSD_GEN_TRC_ELEM_INSTR_RANGE);
-            m_output_elem.setLastInstrInfo(true,m_instr_info.type, m_instr_info.sub_type,m_instr_info.instr_size);
-            m_output_elem.setISA(m_instr_info.isa);
-            resp = outputTraceElementIdx(pElem->getRootIndex(),m_output_elem);
+            resp = outputTraceRange(true, pElem->getRootIndex());
         }
 
         if(m_mem_nacc_pending && OCSD_DATA_RESP_IS_CONT(resp))
@@ -909,7 +914,7 @@ ocsd_datapath_resp_t  TrcPktDecodeEtmV4I::processException()
                  updateContext(pCtxtElem);
 
             // record the exception number
-            m_output_elem.exception_number = pExceptElem->getExcepNum();
+           m_excep_number = pExceptElem->getExcepNum();
 
             // see if there is an implied P0 element on the exception.
             excep_implied_P0 = pExceptElem->getPrevSame();
@@ -970,10 +975,7 @@ ocsd_datapath_resp_t  TrcPktDecodeEtmV4I::processException()
                 break;
                 }
             }
-            m_output_elem.setType(OCSD_GEN_TRC_ELEM_INSTR_RANGE);
-            m_output_elem.setLastInstrInfo(true,m_instr_info.type, m_instr_info.sub_type,m_instr_info.instr_size);
-            m_output_elem.setISA(m_instr_info.isa);
-            resp = outputTraceElementIdx(m_excep_index, m_output_elem);
+            resp = outputTraceRange(true, m_excep_index);
             m_excep_proc = EXCEP_EXCEP;
         }
         else
@@ -984,10 +986,7 @@ ocsd_datapath_resp_t  TrcPktDecodeEtmV4I::processException()
             if(m_output_elem.st_addr != m_output_elem.en_addr)
             {
                 // some trace before we were out of memory access range
-                m_output_elem.setType(OCSD_GEN_TRC_ELEM_INSTR_RANGE);
-                m_output_elem.setLastInstrInfo(true,m_instr_info.type, m_instr_info.sub_type, m_instr_info.instr_size);
-                m_output_elem.setISA(m_instr_info.isa);
-                resp = outputTraceElementIdx(m_excep_index,m_output_elem);
+                resp = outputTraceRange(true, m_excep_index);
             }
 
             m_excep_proc = m_mem_nacc_pending ? EXCEP_NACC : EXCEP_EXCEP;
@@ -1010,6 +1009,7 @@ ocsd_datapath_resp_t  TrcPktDecodeEtmV4I::processException()
         // add end address as preferred return address to end addr in element
         m_output_elem.en_addr = m_excep_addr.val;
         m_output_elem.excep_ret_addr = 1;
+        m_output_elem.exception_number = m_excep_number;
         resp = outputTraceElementIdx(m_excep_index,m_output_elem);  
         m_excep_proc = EXCEP_POP;
     }   
